<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns="http://purl.oclc.org/dsdl/schematron"
              xmlns:g-old="http://genericode.org/2006/ns/CodeList/0.4/"
              xmlns:g="http://docs.oasis-open.org/codelist/ns/genericode/1.0/"
                version="1.0"
                exclude-result-prefixes="g g-old">
<!--
     Support for an implementation of the Schematron-based Value Validation 
     Using Genericode.

     This fragment supports external code list expressions of code list values
     written using either genericode 0.4 or genericode 1.0, where those files
     use only simple values for keys.

     Called template rule entry points:

       code-list-confirm-instance - confirms the referenced value list is
                                    a genericode file
       code-list-make-rule - synthesize a Schematron <rule> based on the
                             meta data and values of the value list
       code-list-get-property-value - return a genericode meta data value based
                                      on the abstract name for a meta data
                                      property

  $Id: genericode-CodeList.xsl,v 1.1 2007/07/28 19:52:59 G. Ken Holman Exp $

Copyright (C) OASIS Open (2006). All Rights Reserved.

This document and translations of it may be copied and furnished to others, and
derivative works that comment on or otherwise explain it or assist in its
implementation may be prepared, copied, published and distributed, in whole or
in part, without restriction of any kind, provided that the above copyright
notice and this paragraph are included on all such copies and derivative works.
However, this document itself may not be modified in any way, such as by
removing the copyright notice or references to OASIS, except as needed for the
purpose of developing OASIS specifications, in which case the procedures for
copyrights defined in the OASIS Intellectual Property Rights document must be
followed, or as required to translate it into languages other than English.

The limited permissions granted above are perpetual and will not be revoked by
OASIS or its successors or assigns.

This document and the information contained herein is provided on an "AS IS"
basis and OASIS DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO ANY WARRANTY THAT THE USE OF THE INFORMATION HEREIN WILL NOT
INFRINGE ANY RIGHTS OR ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE.

-->

<!--========================================================================-->
<!--determine the presence of a document with the expected document element-->
<!--returns the string of an error message if there is a problem-->
<!--
     parameter $valuelist - the supplied declaration of the external value list
     parameter $doc - the root node of the external value list
-->
<xsl:template name="code-list-confirm-instance">
  <xsl:param name="valuelist"/>
  <xsl:param name="doc"/>
  <xsl:choose>
    <xsl:when test="not($doc/g:CodeList) and not($doc/g-old:CodeList)">
      <xsl:text>Resource for value list "</xsl:text>
      <xsl:value-of select="$valuelist/@xml:id"/>
      <xsl:text>" points to the document element {</xsl:text>
      <xsl:for-each select="$doc/*">
        <xsl:value-of select="concat(namespace-uri(.),'}',local-name(.))"/>
      </xsl:for-each>
      <xsl:text> and not to either {</xsl:text>
      <xsl:value-of select="document('')/*/namespace::g"/>
      <xsl:text>}CodeList or {</xsl:text>
      <xsl:value-of select="document('')/*/namespace::g-old"/>
      <xsl:text>}CodeList: </xsl:text>
      <xsl:value-of select="$valuelist/@uri"/>
    </xsl:when>
    <xsl:when test="not($doc/*/ColumnSet/Key)">
      <xsl:text>No column declared as a key in the value list "</xsl:text>
      <xsl:value-of select="$valuelist/@xml:id"/>
      <xsl:text>": </xsl:text>
      <xsl:value-of select="$valuelist/@uri"/>
    </xsl:when>
    <xsl:when test="string($valuelist/@key) and 
                    not($doc/*/ColumnSet/Key[normalize-space(@Id)=
                                           normalize-space($valuelist/@key)])">
      <xsl:text>Declared key "</xsl:text>
      <xsl:value-of select="normalize-space($valuelist/@key)"/>
      <xsl:text>" for value list "</xsl:text>
      <xsl:value-of select="$valuelist/@xml:id"/>
      <xsl:text>" not found: </xsl:text>
      <xsl:value-of select="$valuelist/@uri"/>
    </xsl:when>
    <xsl:when test="string($valuelist/@key) and 
                    count($doc/*/ColumnSet/Key[normalize-space(@Id)=
                            normalize-space($valuelist/@key)]/ColumnRef) != 1">
      <xsl:text>Declared key "</xsl:text>
      <xsl:value-of select="normalize-space($valuelist/@key)"/>
      <xsl:text>" for value list "</xsl:text>
      <xsl:value-of select="$valuelist/@xml:id"/>
      <xsl:text>" is not limited to a single column reference: </xsl:text>
      <xsl:value-of select="$valuelist/@uri"/>
    </xsl:when>
    <xsl:when test="not(string($valuelist/@key)) and
                    count($doc/*/ColumnSet/Key) != 1">
      <xsl:text>No chosen key for value list "</xsl:text>
      <xsl:value-of select="$valuelist/@xml:id"/>
      <xsl:text>" that references a list containing multiple keys: </xsl:text>
      <xsl:value-of select="$valuelist/@uri"/>
    </xsl:when>
    <xsl:when test="not(string($valuelist/@key)) and
                    count($doc/*/ColumnSet/Key/ColumnRef) != 1">
      <xsl:text>Default key for value list "</xsl:text>
      <xsl:value-of select="$valuelist/@xml:id"/>
      <xsl:text>" is not limited to a single column reference: </xsl:text>
      <xsl:value-of select="$valuelist/@uri"/>
    </xsl:when>
  </xsl:choose>
</xsl:template>

<!--========================================================================-->
<!--build Schematron <rule> elements using the selection of available values-->
<!--note the liberal use of newline sequences in the result using <xsl:text>
    in order to make the end result legible to a reader; these have no effect
    on the well-formedness of the result or the Schematron validation-->
<!--
     parameter $item - the XPath qualified name of the item being tested
     parameter $context - the XPath address of the item being tested
     parameter $values-identifier - normalized values= attribute
     parameter $valuelists-referenced - the node set of <ValueList> elements
     parameter $assoc-uri - the URI of the file with this association
-->
<xsl:template name="code-list-make-rule">
  <xsl:param name="item"/>
  <xsl:param name="context"/>
  <xsl:param name="values-identifier"/>
  <xsl:param name="valuelists-referenced"/>
  <xsl:param name="assoc-uri"/>
  <xsl:variable name="assertion">
    <xsl:for-each select="$valuelists-referenced">
      <xsl:variable name="list" select="."/>
      <xsl:variable name="key" select="normalize-space(@key)"/>
      <xsl:variable name="values" 
                select="document(@uri,.)/*/SimpleCodeList/Row/Value
                                         [@ColumnRef=../../../ColumnSet/
                                          Key[( $key and @Id=$key ) or 
                                              ( not( $key ) and position()=1 )]
                                          /ColumnRef/@Ref]/SimpleValue"/>
      <xsl:if test="$values">
        <!--this list is tacked on to the others; first gets trimmed later-->
        <xsl:text> or ( ( </xsl:text>
        <!--check for the need for meta data in the comparison-->
        <xsl:for-each select="document(@uri,.)/*">
          <xsl:variable name="metadata">
            <!--check properties based on item's name stem-->
            <xsl:call-template name="build-metadata-comparison-string">
              <xsl:with-param name="item" select="$item"/>
              <xsl:with-param name="list" select="$list"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:if test="string($metadata)">
            <!--only include metadata if there are comparisons-->
            <xsl:value-of select="concat($metadata,' ) and ( ')"/>
          </xsl:if>
        </xsl:for-each>
        <!--values of information item for this particular context-->
        <xsl:choose>
          <xsl:when test="not($values[contains(.,' ')])">
            <!--none of the code values have any embedded spaces-->
            <xsl:text>not(contains(normalize-space(.),' '))</xsl:text>
            <xsl:text> and contains( ' </xsl:text>
            <xsl:for-each select="$values/../..">
              <!--walk through the rows-->
              <xsl:variable name="row" select="."/>
              <xsl:if test="position()>1">
                <xsl:text> </xsl:text>
              </xsl:if>
              <!--build the value from the columns in columnref order-->
              <xsl:for-each select="../../ColumnSet/
                                    Key[( $key and @Id=$key ) or 
                                        ( not( $key ) and position()=1 )]/
                                    ColumnRef">
                <xsl:variable name="keycolref" select="@Ref"/>
                <xsl:for-each select="$row/Value[normalize-space(@ColumnRef)=
                                                 normalize-space($keycolref)]/
                                      SimpleValue">
                  <xsl:call-template name="check-value-validity">
                    <xsl:with-param name="list" select="$list"/>
                  </xsl:call-template>
                  <xsl:value-of select="."/>
                </xsl:for-each>
              </xsl:for-each>
            </xsl:for-each>
           <xsl:text> ',concat(' ',normalize-space(.),' ') )</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <!--at least one code value has an embedded space-->
            <xsl:for-each select="$values/../..">
              <!--walk through the rows-->
              <xsl:variable name="row" select="."/>
              <xsl:if test="position()>1"> or </xsl:if>
              <xsl:text>. = '</xsl:text>
              <!--build the value from the columns in columnref order-->
              <xsl:for-each select="../../ColumnSet/
                                    Key[( $key and @Id=$key ) or 
                                        ( not( $key ) and position()=1 )]/
                                    ColumnRef">
                <xsl:variable name="keycolref" select="@Ref"/>
                <xsl:for-each select="$row/Value[normalize-space(@ColumnRef)=
                                                 normalize-space($keycolref)]/
                                      SimpleValue">
                  <xsl:call-template name="check-value-validity">
                    <xsl:with-param name="list" select="$list"/>
                  </xsl:call-template>
                  <xsl:value-of select="."/>
                </xsl:for-each>
              </xsl:for-each>
              <xsl:text>'</xsl:text>
            </xsl:for-each>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text> ) )</xsl:text>
      </xsl:if>
    </xsl:for-each>
  </xsl:variable>
  <!--only add a rule for item if there are values against which to compare-->
  <xsl:if test="string($assertion)">
    <xsl:text>
</xsl:text>
    <rule context="{$context}"><xsl:text>
  </xsl:text>
      <!--indicate values that must be true-->
      <!--the first " or " in the string is not required, so trim it off-->
      <assert test="{substring($assertion,5)}">
        <!--message to report if the test fails-->
        <xsl:call-template name="compose-violation-report">
         <xsl:with-param name="values-identifier" select="$values-identifier"/>
         <xsl:with-param name="assoc-uri" select="$assoc-uri"/>
         <xsl:with-param name="context" select="$context"/>
        </xsl:call-template>
      </assert><xsl:text>
</xsl:text>
    </rule><xsl:text>
</xsl:text>
  </xsl:if>
</xsl:template>

<xsl:template name="check-value-validity">
  <xsl:param name="list"/>
  <!--in support of above, this is broken out because it has no variables-->
  <xsl:if test="contains(.,&#34;'&#34;)">
    <xsl:message terminate="yes">
      <xsl:text>This methodology does not support key values </xsl:text>
      <xsl:text>containing single quotes: "</xsl:text>
      <xsl:value-of select="."/>
      <xsl:text>" in list: xmlid=</xsl:text>
      <xsl:value-of select="$list/@xml:id"/>
    </xsl:message>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<!--
     Return the code list's metadata value string for the requested property.
     Return the empty string if no such property is available.

     genericode <Identification> child elements:

       property Reference   ==  ShortName
       property Name        ==  LongName
       property ID          ==  LongName[@Identifier='listID'] or if not
                                present, use LongName
       property URI         ==  CanonicalUri
       property Version     ==  Version  
       property VersionURI  ==  CanonicalVersionUri  
       property LocationURI ==  LocationUri
       property AgencyName  ==  Agency/LongName
       property AgencyID    ==  Agency/Identifier

     Parameter $property - the meta data identifier to obtain from the list
     Current node        - document element of the referenced list
-->

<xsl:template name="code-list-get-property-value">
  <xsl:param name="property"/>

  <xsl:variable name="property-value">
    <xsl:choose>
      <xsl:when test="$property='Reference'">
        <xsl:value-of select="Identification/ShortName[1]"/>
      </xsl:when>
      <xsl:when test="$property='Name'">
        <xsl:value-of select="Identification/LongName[1]"/>
      </xsl:when>
      <xsl:when test="$property='ID'">
        <xsl:choose>
          <xsl:when test="Identification/LongName[@Identifier='listID']">
            <xsl:value-of 
                  select="Identification/LongName[@Identifier='listID'][1]"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="Identification/LongName[1]"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="$property='URI'">
        <xsl:value-of select="Identification/CanonicalUri[1]"/>
      </xsl:when>
      <xsl:when test="$property='Version'">
        <xsl:value-of select="Identification/Version[1]"/>
      </xsl:when>
      <xsl:when test="$property='VersionURI'">
        <xsl:value-of select="Identification/CanonicalVersionUri[1]"/>
      </xsl:when>
      <xsl:when test="$property='LocationURI'">
        <xsl:value-of select="Identification/LocationUri[1]"/>
      </xsl:when>
      <xsl:when test="$property='AgencyName'">
        <xsl:value-of select="Identification/Agency[1]/LongName[1]"/>
      </xsl:when>
      <xsl:when test="$property='AgencyID'">
        <xsl:value-of select="Identification/Agency[1]/Identifier[1]"/>
      </xsl:when>
    </xsl:choose>
  </xsl:variable>
  <xsl:value-of select="normalize-space($property-value)"/>
</xsl:template>

</xsl:stylesheet>
